#version 330

layout(location = 0) out vec4 fragColor;
layout(location = 1) out vec4 normalVector;
layout(location = 2) out vec4 view_pos;

in vec4 positionFrag;
in vec4 normalFrag;
in vec4 viewPositionFrag;
in vec3 normalVFrag;
in vec2 uvFrag;

// coordinates in the ShadowMap
in vec4 SM_tex_coord;

// material information
uniform vec3 ambientFactor;
uniform vec3 diffuseFactor;
uniform vec3 specularFactor;
uniform float shininess;

uniform float near_plane;
uniform float far_plane;

// light information
uniform vec3 lightDirection;
uniform vec3 lightColor;
uniform float lightIntensity;

// the camera's position
uniform vec4 cameraPosition;

// the object's texture
uniform sampler2D colorTexture;

// shadow map
uniform sampler2D shadowMap;

float linearDepth(float depth)
{
      return (2 * near_plane) / (far_plane + near_plane - depth * (far_plane - near_plane));
}

void main()
{
	vec4 color = texture(colorTexture, uvFrag);

	if (color.w < 0.9)
		discard;

	///// Shadow Mapping /////
	/*
	float visibility = 1.0;

	if (SM_tex_coord.w > 0.0) {
		vec3 SM_position = SM_tex_coord.xyz / SM_tex_coord.w;

		if (texture(shadowMap, SM_position.xy).x  <  (SM_position.z - 0.005)) {
			visibility = 0.0;
		}
	}
	*/

	///// Variance Shadow Mapping /////

	float visibility = 1.0;

	if (SM_tex_coord.w > 0.0) {
		vec3 SM_position = SM_tex_coord.xyz / SM_tex_coord.w;

		vec2 moments = texture(shadowMap, SM_position.xy).rg;

		if (moments.x < SM_position.z -  0.005) {
			float variance = max(moments.y - (moments.x * moments.x), 0.005);
			float diff = SM_position.z - moments.x;

			visibility = variance / (variance + diff * diff);
			visibility = smoothstep(0.6, 1.0, visibility);
		}
	}


	///// Normal Vector /////

	normalVector = vec4(normalVFrag, 1);


	///// View Pos/////

	view_pos = viewPositionFrag;//vec4(vec3(linearDepth(gl_FragCoord.z)), 1);


	///// Blinn Phong /////

	vec4 lightDir = vec4(normalize(lightDirection), 0.0);
	vec4 viewVector = normalize(positionFrag - cameraPosition);

	vec4 reflectedLightVector = reflect(lightDir, normalFrag);

	float k_diffuse = max(dot(normalFrag, lightDir), 0.0);
	float k_specular = max(dot(viewVector, reflectedLightVector), 0.0);

	vec3 diffuseColor = k_diffuse * diffuseFactor;
	vec3 specularColor = pow(k_specular, shininess) * specularFactor * k_diffuse;

	vec3 intensity = ambientFactor + (diffuseColor + specularColor) * lightIntensity * visibility;

	fragColor = vec4(color.rgb * lightColor * intensity, color.w);
}